program test

#include "petsc/finclude/petscdmplex.h"
#include "petsc/finclude/petscdmlabel.h"
 use petscdmplex
 use petscsys

 implicit none

 DM                                :: dm, dm_orig, dm_geom
 DMLabel                           :: label
 DMLabel                           :: label_cell
 DMLabel                           :: label_FaceSets
 DMLabel                           :: label_boundary
 DMLabel                           :: label_inlet
 DMLabel                           :: label_outlet
 DMLabel                           :: label_upper
 DMLabel                           :: label_lower

 DMLabel, pointer                  :: nolabel(:) => NULL()
 Vec                               :: coord
 Vec                               :: x
 Vec                               :: y
 Vec                               :: vel
 PetscInt                          :: x_size
 PetscInt                          :: y_size
 PetscInt                          :: vel_size
 PetscInt                          :: est, eend ! start & end range for edges
 PetscInt                          :: pst, pend ! start & end range for edges
 PetscInt                          :: dim, val, depth
 PetscInt, parameter               :: marker_inlet  = 5
 PetscInt, parameter               :: marker_outlet = 6
 PetscInt, parameter               :: marker_upper  = 7
 PetscInt, parameter               :: marker_lower  = 8

 PetscInt, allocatable             :: marker_list(:)

 PetscInt                          :: i
 PetscScalar                          :: one
 PetscMPIInt                       :: myid, nproc
 PetscReal                         :: vol
 PetscInt, parameter               :: zero = 0

 ! From Example : CreateSection Not working ...
 PetscInt                          :: numFields = 3
 PetscInt                          :: numBC     = 1

 PetscInt, target, dimension(3)    :: numComp
 PetscInt, target, dimension(12)   :: numDof
 PetscInt, pointer                 :: pNumComp(:)
 PetscInt, pointer                 :: pNumDof(:)

 PetscInt, target, dimension(1)    :: bcField
 PetscInt, pointer                 :: pBcField(:)

 IS, target, dimension(1)          :: bcCompIS
 IS, target, dimension(1)          :: bcPointIS

 IS, pointer                       :: pBcCompIS(:)
 IS, pointer                       :: pBcPointIS(:)
 ! From Example : CreateSection Not working ...

 PetscReal, pointer                :: pcent(:)
 PetscReal, pointer                :: pnorm(:)
 PetscBool                         :: Interpolate = PETSC_TRUE
 PetscViewer                       :: viewer
 PetscSection                      :: section
 PetscSection                      :: coordSection
 PetscErrorCode                    :: ierr
 character(len=PETSC_MAX_PATH_LEN) :: OutStr
 character(len=PETSC_MAX_PATH_LEN) :: fname

 integer                           :: num


 ! initialize petsc : This section also initializes MPI
 call PetscInitialize(PETSC_NULL_CHARACTER, ierr)
 if(ierr.ne.0) then
   write(*,*) 'Fail to initialize PETSc ...'
   stop
 endif

 ! get MPI size info 
 call MPI_COMM_RANK(MPI_COMM_WORLD, myid, ierr)
 call MPI_COMM_SIZE(MPI_COMM_WORLD, nproc, ierr)
 write(*,*) 'MPI information ... ', myid, nproc

 fname = './new.msh'
 call DMPlexCreateGmshFromFile(PETSC_COMM_WORLD, fname, Interpolate, dm_orig, ierr);CHKERRA(ierr)
 if(nproc .gt. 1) then
   call DMPlexDistribute(dm_orig, 0, PETSC_NULL_SF, dm, ierr);CHKERRA(ierr)
 else
   call DMClone(dm_orig, dm, ierr);CHKERRA(ierr)
 endif

 call DMPlexGetChart(dm, pst, pend, ierr);CHKERRA(ierr) 

 call DMPlexGetHeightStratum(dm, 1, est, eend, ierr);CHKERRA(ierr) ! edges

 call DMGetLabel(dm, "Face Sets", label_FaceSets, ierr);CHKERRA(ierr)

 ! Identify physical boundary & put label
 call DMCreateLabel(dm, "inlet", ierr);CHKERRA(ierr)
 call DMCreateLabel(dm, "outlet", ierr);CHKERRA(ierr)
 call DMCreateLabel(dm, "upper", ierr);CHKERRA(ierr)
 call DMCreateLabel(dm, "lower", ierr);CHKERRA(ierr)
 do i=est, eend
   call DMLabelGetValue(label_FaceSets, i, val, ierr);CHKERRA(ierr)
   if(val.eq.marker_inlet)  call DMSetLabelValue(dm, "inlet",  i, val, ierr);CHKERRA(ierr)
   if(val.eq.marker_outlet) call DMSetLabelValue(dm, "outlet", i, val, ierr);CHKERRA(ierr)
   if(val.eq.marker_upper)  call DMSetLabelValue(dm, "upper",  i, val, ierr);CHKERRA(ierr)
   if(val.eq.marker_lower)  call DMSetLabelValue(dm, "lower",  i, val, ierr);CHKERRA(ierr)
 enddo
 call DMGetLabel(dm, "inlet",  label_inlet,  ierr);CHKERRA(ierr)
 call DMGetLabel(dm, "outlet", label_outlet, ierr);CHKERRA(ierr)
 call DMGetLabel(dm, "upper",  label_upper,  ierr);CHKERRA(ierr)
 call DMGetLabel(dm, "lower",  label_lower,  ierr);CHKERRA(ierr)


 ! PetscSection
 call PetscSectionCreate(PETSC_COMM_WORLD, section, ierr);CHKERRA(ierr)
 call PetscSectionSetNumFields(section, numFields, ierr);CHKERRA(ierr)
 call PetscSectionSetChart(section, pst, pend, ierr);CHKERRA(ierr)
 do i=pst,pend-1 
   call PetscSectionSetDof(section, i, 1, ierr);CHKERRA(ierr)
 enddo
 call PetscSectionSetUp(section, ierr);CHKERRA(ierr)
 ! name the field variables
 call PetscSectionSetFieldName(section, 0, 'x', ierr);CHKERRA(ierr)
 call PetscSectionSetFieldName(section, 1, 'y', ierr);CHKERRA(ierr)
 call PetscSectionSetFieldName(section, 2, 'vel', ierr);CHKERRA(ierr)
 call PetscSectionView(section, PETSC_VIEWER_STDOUT_WORLD, ierr);CHKERRA(ierr)

 ! tell the DM to use this data layout
 call DMSetLocalSection(dm, section, ierr);CHKERRA(ierr)

 ! create a Vec with this layout and view it
 call DMGetGlobalVector(dm, x, ierr);CHKERRA(ierr)
 call DMGetGlobalVector(dm, y, ierr);CHKERRA(ierr)
 call DMGetGlobalVector(dm, vel, ierr);CHKERRA(ierr)

 ! create viewer
 call PetscViewerCreate(PETSC_COMM_WORLD, viewer, ierr);CHKERRA(ierr)

 ! set viewer type
 call PetscViewerSetType(viewer, PETSCVIEWERVTK, ierr);CHKERRA(ierr)

 ! set output file name for viewer
 call PetscViewerFileSetName(viewer, './output.vtk', ierr);CHKERRA(ierr) 

 ! print dm object to the file
 call DMView(dm, viewer, ierr);CHKERRA(ierr)
 call VecView(x, viewer, ierr);CHKERRA(ierr)
 call VecView(y, viewer, ierr);CHKERRA(ierr)

 ! destroy
 call PetscViewerDestroy(viewer, ierr);CHKERRA(ierr)

 call DMRestoreGlobalVector(dm, x, ierr);CHKERRA(ierr)
 call DMRestoreGlobalVector(dm, y, ierr);CHKERRA(ierr)

 call PetscSectionDestroy(section, ierr);CHKERRA(ierr)

 call DMDestroy(dm, ierr);CHKERRA(ierr)
 call DMDestroy(dm_orig, ierr);CHKERRA(ierr)
 call DMDestroy(dm_geom, ierr);CHKERRA(ierr)

 ! finalize petsc
 write(OutStr,*) 'done ...', myid, '\n'
 call PetscPrintf(PETSC_COMM_SELF, OutStr, ierr);CHKERRA(ierr) 
 call PetscFinalize(ierr)

 stop
end program test
